// MgInit.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGINIT_H
#define MGINIT_H

#include "MgAudioSystem.h"
#include "MgVideoSystem.h"
#include "MgTimerSystem.h"
#include "MgNetSystem.h"
#include "MgApp.h"

#include "MgBeginCode.h"

/*! \file MgInit.h

	This file includes the alternate libMgTk initialization routines.
*/

/*! The systems class returned by MgTkInit(). */
class MGDLL MgSystems
{
public:
	/*! Get a pointer to the app. */
	MgApp * App() const { return fApp; }
	/*! Video subsystem. */
	MgVideoSystem * Video() const { return fVideo; }
	/*! Audio subsystem. */
	MgAudioSystem * Audio() const { return fAudio; }
	/*! Timer subsystem. */
	MgTimerSystem * Timer() const { return fTimer; }
	/*! Net subsystem. */
	MgNetSystem * Net() const { return fNet; }

private:
	MgVideoSystem * fVideo;
	MgAudioSystem * fAudio;
	MgTimerSystem * fTimer;
	MgNetSystem * fNet;

	MgApp * fApp;
	
	/* MgSystems operator delete, overridden to prevent user deletion. */
	void operator delete(void * p)
	{
		::delete(p);
	}
	
	friend MGDLL const MgSystems * const MgTkInit(Uint32);
	friend MGDLL void MgTkQuit();
};

#define MG_INIT_FAILED 0		/* equivalent to NULL */
#define MG_INIT_RUNNING ((const MgSystems * const)-1)
#define MG_INIT_WRONG_SDL ((const MgSystems * const)-2)

/*! This is an alternate way to use libMgTk. Use this only if you do not want to
	derive from an MgApp and run a fully multi-threaded app, or if you just
	don't want to use the architecture provided by the MgApp implementation. But be
	warned, the default implementation of MgApp doesn't DO anything (with the
	exception of MgApp::GetRawEvent() and MgApp::WaitRawEvent()).
	
	Remember: if you use this method, you are responsible for starting all subsystems
	via the subsystem pointers returned by MgTkInit(), as well as calling MgTkQuit().
	This function calls SDL_Init() and SDL_Quit() for you.
	
	\note If you used MgTkInit() to start libMgTk, you can still use event-filter based
		  classes, but these events will no longer run in a seperate thread because
		  you have to call MgApp::PumpEvents() occasionally to run the event filters.

	\param flags One or more of the flags declared in MgApp.h
	
	\retval MG_INIT_FAILED Out of memory or SDL_Init() failed.
	\retval MG_INIT_RUNNING libMgTk is already running due to an MgApp being created.
	\retval MG_INIT_WRONG_SDL Wrong version of SDL installed. 
	\retval Other libMgTk was initialized successfully.
*/
MGDLL const MgSystems * const MgTkInit(Uint32 flags);

/*! Quit libMgTk. */
MGDLL void MgTkQuit();

#include "MgEndCode.h"

#endif

