// MgKeyboard.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/
#ifndef MGKEYBOARD_H
#define MGKEYBOARD_H

/*! \file MgKeyboard.h

	Contains the keyboard handling code.
	
	\note When getting the state of keys, use SDL's SDLKey enum
		  to do so. This enum is too large to duplicate in libMgTk.
*/

#include "MgEventFilter.h"

#include "MgBeginCode.h"

//! This mask refers to all hotkey bindings.
#define MG_ALLHOTKEYS SDL_ALL_HOTKEYS

//! Typdef SDLKey to MgKey. There is no difference between the two.
typedef SDLKey MgKey;

//! Typedef SDLMod to MgMod. There is no difference between the two.
typedef SDLMod MgMod;

//! Default repeat delay.
#define MG_DEFAULT_REPEAT_DELAY SDL_DEFAULT_REPEAT_DELAY
//! Default repeat interval.
#define MG_DEFAULT_REPEAT_INTERVAL SDL_DEFAULT_REPEAT_INTERVAL

/*! This is the event-based keyboard class. It also includes
	some static methods for dealing with keyboards. There is no
	static version of this class because all you have to do
	to get updated input at demand is call the static member
	MgKeyboard::GetKeyState().
*/
class MGDLL MgKeyboard : public MgEventFilter
{
public:
	//! Constructor.
	MgKeyboard();
	//! Destructor.
	virtual ~MgKeyboard();
	
	//! Enable/disable UNICODE translation of keyboard input.
	static bool EnableUNICODE(bool enable);

	//! Enable/disable keyboard repeat. Default is "off".
	static int EnableKeyRepeat(int delay, int interval);

	//! Get a snapshot of the current state of the keyboard. You do not really need to use this yourself.
	static Uint8 * GetKeyState(int * numkeys = NULL);
	
	//! Get the current modifier state.
	static MgMod GetModState();
	
	//! Set the modifier state.
	static void SetModState(MgMod modstate);
	
	//! Get the name of keysym.	Do not modify the returned value in any way!
	static char * GetKeyName(MgKey key);
	
	//! Event type.
	enum Type
	{
		Up = 1,		//!< Key released.
		Down		//!< Key pressed.
	};
	
	//! Call this to see if the object was constructed properly. Returns false if not.
	bool InitCheck() const;
	
	virtual bool Keyboard(const MgKeyboardEvent & e);

protected:
	//! Override this method to catch the event.
	virtual void Activated(MgKeyboard::Type code) = 0;

	//! Contains the key event data.
	MgKeyboardEvent fKeyEvent;
	
private:
	bool fOK;
};
	
#include "MgEndCode.h"

#endif

