// MgPalette.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGPALETTE_H
#define MGPALETTE_H

#include <SDL_video.h>

/*! \file MgPalette.h
	Includes the palette routines for libMgTk's video routines.
*/
#include "MgColor.h"
#include "MgBeginCode.h"

class MgSurface;
class MgPainter;
class MgScreen;

/*!
	This class provides a simple way to set/add/remove colors from the
	palette of a surface. The most colors a palette can contian is 256.
*/
class MGDLL MgPalette
{
public:
	//! Constructor.
	MgPalette();
	//! Constructor.
	MgPalette(SDL_Surface * surface);
	//! Constructor.
	MgPalette(MgSurface * surface);
	//! Destructor.
	~MgPalette();
	
	//! Sets a color.
	bool SetColor(int index, const MgColor & color);
	//! Add a color.
	bool AddColor(const MgColor & color);
	//! Get a color.
	MgColor GetColor(int index);

	//! Number of colors in the palette.
	int NumColors() const;
	//! Set the size of the palette, removing any that are over that amount.
	void SetSize(int size);
	
private:
	void ApplyTo(MgSurface * surface);
	void ApplyTo(SDL_Surface * surface);
	
	MgColor * fColors;
	int fNumColors;
	
	friend class MgSurface;
	friend class MgPainter;
	friend class MgScreen;
};

#include "MgEndCode.h"

#endif

