// MgSound.h
//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef MGSOUND_H
#define MGSOUND_H

/*! \file MgSound.h
	This file contains the base class for all sound classes.
*/
#include <SDL_types.h>

#include "MgBeginCode.h"

/*! With this class, you can write your own audio loading routines for
	other formats not included with libMgTk. Just implement the required
	functions.
*/
class MGDLL MgSound
{
public:
	//! Constructor. Does nothing.
	MgSound() {}
	//! Destructor. Does nothing.
	virtual ~MgSound() {}
	
	/*! This function is called when you need to mix your audio with the
		audio stream.
	*/
	virtual void Mix(Uint8 * stream, int len, int volume) = 0;
	/*! You should keep an internal int variable keeping track of the
		position at which the sound is at (in bytes) and return it
		in this method. This variable should be updated in Mix().
	*/
	virtual int SoundPos() const = 0;
	/*! Return the length of the sound here! */
	virtual int SoundLength() const = 0;
	/*! The MgMixer calls this if this sound is to be looped. You should
		reset your variable that keeps track of the amount of the sound
		is left to the length of the sound.
	*/
	virtual void Rewind() = 0;
	/*! This function needs to return a raw pointer to the allocated audio
		stream.
	*/
	virtual Uint8 * Sound() const = 0;
	
private:
};

#include "MgEndCode.h"

#endif

