//-----------------------------------------------------------------------------
/*
    libMgTk - Mage Toolkit
    Copyright (C) 2001 Vitaliy Mikitchenko (Mage Productions)

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    Vitaliy Mikitchenko
    webmaster@magep.cjb.net
*/

#ifndef DEBUG_NEW_H
#define DEBUG_NEW_H

#ifdef DEBUG

#include <string.h>
#include <list>
#include <stdio.h>

struct _dbg_info_t
{
	int addr;
	size_t size;
	char file[512];
	int line;
};

typedef std::list<_dbg_info_t *> _dbg_list_t;

static _dbg_list_t * _dbg_list = NULL;

void _dbg_add(int addr, size_t size, const char * file, int line);
void _dbg_rem(int addr);

inline void *
operator new(size_t size, const char * file, int line)
{
	void * p = malloc(size);

	_dbg_add((int)p, size, file, line);
	return p;
}

inline void
operator delete(void * p)
{
	_dbg_rem((int)p);
	free(p);
}

inline void *
operator new[](size_t size, const char * file, int line)
{
	void * p = malloc(size);

	_dbg_add((int)p, size, file, line);
	return p;
}

inline void
operator delete[](void * p)
{
	_dbg_rem((int)p);
	free(p);
}

inline void
_dbg_add(int addr, size_t size, const char * file, int line)
{
	_dbg_info_t * info;
	
	if (!_dbg_list)
		_dbg_list = new _dbg_list_t;	// have to use new
	
	info = (_dbg_info_t *)malloc(sizeof(_dbg_info_t));;
	info->addr = addr;
	info->size = size;
	info->line = line;
	//info->file = (char *)malloc(strlen(file) + 1);	// use malloc instead of "new"
	strncpy(info->file, file, 511);
	
	if (strlen(file) > 511)
		info->file[511] = 0;
	else
		info->file[strlen(file) - 1] = 0;
	
	_dbg_list->insert(_dbg_list->begin(), info);
}

inline void
_dbg_rem(int addr)
{
	_dbg_list_t::iterator i;
	
	if (!_dbg_list)
		return;
	
	for (i = _dbg_list->begin(); i != _dbg_list->end(); i++)
	{
		if ((*i)->addr == addr)
		{
			_dbg_list->remove((*i));
			break;
		}
	}
}

/* This is the only function you should ever call yourself. Call it
 * right before you exit the program.
 */
inline void
_dbg_dump_unfreed()
{
	_dbg_list_t::iterator i;
	size_t tsize = 0;
	
	if (!_dbg_list)
	{
		printf("Nothing was ever allocated with \"new\"\n");
		return;
	}
	
	for (i = _dbg_list->begin(); i != _dbg_list->end(); i++)
	{
		printf("%s:\t\tLINE %d\t\tADDRESS 0x%.8x\t\t%ld bytes unfreed\n",
				(*i)->file, (*i)->line, (*i)->addr, (*i)->size);
		tsize += (*i)->size;
	}
	printf("------------------------------------------------------------------------------------------------------\n");
	printf("Total unfreed: %ld bytes\n", tsize);
};
#else
inline void
_dbg_dump_unfreed()
{}
#endif /* DEBUG */


#ifdef DEBUG	// yes... another ifdef...
#define new new(__FILE__, __LINE__)
//#else
//#define DEBUG_NEW new
#endif

//#define 

#endif

